(ns wanijo.attribute.domain
  (:require [clojure.spec.alpha :as spec]
            [wanijo.framework.neo4j :as neo4j]))

(spec/def ::name
  (spec/and string? not-empty))

(spec/def ::type
  (spec/and
   string?
   #(some #{"date" "file" "string" "text"} %)))

(spec/def ::required
  (spec/or :bool-val boolean?
           :nil nil?))

(neo4j/defquery
  findy-by-schema
  "MATCH (a:attribute)-->(s:schema)
   WHERE s.uuid = {uuid}
   RETURN a
   ORDER BY a.name")

(defn find-by-schema! [schema-uuid]
  (->>
   (neo4j/exec-query!
    findy-by-schema
    {:uuid schema-uuid})
   (map :a)))

(neo4j/defquery create-new
  "MATCH (u:user)--(s:schema)
   WHERE u.uuid = {user_uuid}
     AND s.uuid = {schema_uuid}
   CREATE (a:attribute)
   CREATE (a)-[:created_by]->(u)
   CREATE (a)-[:of]->(s)
      SET a.type = {type}
      SET a.name = {name}")

(defn create-new! [name type schema-uuid user-uuid]
  (neo4j/exec-query!
   create-new
   {:user_uuid user-uuid
    :schem_uuid schema-uuid
    :type type
    :name name}))
